
function depth = defocus(name, image_num1, image_num2, render_3d)
    global resize_size;
    global debug_on;
    global show_images;
    global kernel_size;
    resize_size = 300;
    kernel_size = 16;
    debug_on = 1;
    show_images = 1;

    image_data(1) = load_image_data( ['images/' name num2str(image_num1)] );
    image_data(2) = load_image_data( ['images/' name num2str(image_num2)] );
    
    colors = image_data(1).image;
    
    depth = stm1(image_data);
    
    if nargin > 3 && render_3d
        show_shape(depth, colors);
    end
end


%%
function depth_map = stm1(image_data)
    global kernel_size;
    kern_size = kernel_size;

    %rescale one image for magnification
    image1 = normalize_and_greyscale(image_data(1).image);
    image2 = normalize_and_greyscale(image_data(2).image);
    print_matrix(image1, 'normalized1'); print_matrix(image2, 'normalized2');
    show_image(image1, 'normalized1');
    show_image(image2, 'normalized2');
    
    % smooth and take the laplacian
    blur = smoothing_coefficients(kern_size);
    smooth1 = apply_separable_filter(image1, blur);
    smooth2 = apply_separable_filter(image2, blur);
    show_image(smooth1, 'smooth g_1');
    show_image(smooth2, 'smooth g_2');
    
    lapl = laplacian_coefficients(kern_size);
    laplacian1 = apply_separable_filter(image1, lapl);
    laplacian2 = apply_separable_filter(image2, lapl);
    
    print_matrix(laplacian1, 'laplacian g_1'); print_matrix(laplacian2, 'laplacian g_2');
    mean_laplacian = (laplacian1 + laplacian2) ./ 2;
    show_image(laplacian1, '\nabla g_1');
    show_image(laplacian2, '\nabla g_2');
    show_image(mean_laplacian, '\nabla g');
    
    Gsquared = estimate_Gsquared(smooth1, smooth1, smooth2);
    print_matrix(Gsquared, 'G^2');
    show_image(Gsquared, 'integrate G^2');
    
    G = sqrt(Gsquared);
    print_matrix(G, 'G');
    show_image(G, 'integrate G');

    v1 = variance(image1);
    v2 = variance(image2);
    show_image(v1, 'variance v_1');
    show_image(v2, 'variance v_2');
    sign_of_G = blurrier(v1, v2);
    print_matrix(sign_of_G, 'sign from var');
    show_image(sign_of_G, 'variance sign from var');
    
    Gprime = G;
    Gprime = sqrt(G(:,:).^2);
    Gprime = Gprime .* double(sign_of_G);
    print_matrix(Gprime, 'G''');
    show_image(Gprime, 'integrate G''');
    
    m1 = calc_m(image_data(1));
    m2 = calc_m(image_data(2));
    c1 = calc_c(image_data(1));
    c2 = calc_c(image_data(2));
    alpha = calc_alpha(m1, m2);
    beta = calc_beta(m1, m2, c1, c2);
    print_image_data(image_data(1));
    print_image_data(image_data(2));
    fprintf('camera calcs: m1=%.2f m2=%.2f c1=%.2f c1=%.2f a=%.2f B=%.2f\n\n', ...
        m1,m2,c1,c2,alpha,beta);
    
    if image_data(1).focus_dis ~= image_data(2).focus_dis
        blur_radius2 = blur_from_focus(Gprime, beta);
    else
        blur_radius2 = blur_from_aperture(Gprime, alpha);
    end
    blur_radius1 = estimate_image1_blur(blur_radius2, alpha, beta);
    print_matrix(blur_radius1, 'blur sigma_1');
	print_matrix(blur_radius2, 'blur sigma_2');
    show_image(blur_radius1, 'blur \sigma_1');
    show_image(blur_radius2, 'blur \sigma_2');
    
    %calc histogram
    blur_radius2 = remove_outliers(blur_radius2, kern_size);
    
    
    depth_map = depth_from_blur(m2, c2, blur_radius2);
    print_matrix(depth_map, 'depthmap');
    show_image(depth_map, 'depthmap');
end

%%
% helper functions


function im = apply_separable_filter(image, filter)
    im = imfilter(image, filter, 'replicate');
    im = imfilter(im, filter', 'replicate');
end

function h = smoothing_coefficients(filter_size)
    h = zeros(1, filter_size*2 + 1);
    for k=-filter_size:filter_size
        n = k;
        N = filter_size;
        numer = 3 * (5*n^2 - (3*N^2 +3*N - 1));
        denom = (2*N-1) * (2*N+1) * (2*N+3);
        s = -numer / denom;
        h(k+filter_size+1) = s;
    end
end

function h = laplacian_coefficients(filter_size)
    h = zeros(1, filter_size*2 + 1);
    for k=-filter_size:filter_size
        n = k;
        N = filter_size;
        numer = 30 * (3*n^2 - N*(N+1));
        denom = N * (N+1) * (2*N-1) * (2*N+1) * (2*N+3);
        s = -numer / denom;
        h(k+filter_size+1) = s;
    end
end

function v = variance(image)
    overall_mean = mean(mean(image));
    v = (image(:,:) - overall_mean) .^ 2;
    print_matrix(v, 'variance g-[]');    
end

function b = blurrier(variance1, variance2)
    diff = variance2 - variance1;
    show_image(normalize_and_greyscale(diff), 'variance v_2 - v_1');
    b = diff(:,:) ./ sqrt(diff(:,:).^2);
end

function g = estimate_Gsquared(mean_laplacian, g1, g2)
    global kernel_size;
    kern = ones(kernel_size,kernel_size);
    
    numerator = (g1 - g2) .^ 2;
    show_image(numerator, 'integrate (g_1-g_2)^2');
    denominator = mean_laplacian .^ 2;
    %denominator = g1 .^ 2;    %typo in paper?
    show_image(denominator, 'integrate \nabla g^2');

    numerator_integrated = imfilter(numerator, kern, 'replicate');
    denominator_integrated = imfilter(denominator, kern, 'replicate');
    show_image(numerator_integrated, 'integrate \int{(g_1-g_2)^2}');
    show_image(denominator_integrated, 'integrate \int{\nabla g^2}');
    
    g = (numerator_integrated ./ denominator_integrated) .* 16;
end

function s = blur_from_focus(Gprime, beta)
    s = (Gprime - beta^2) ./ (2 * beta);
end

function s = blur_from_aperture(Gprime, alpha)
    %should be +/-, but I'm assuming plus since the image order is know
    %(image 2 is farther)
    s = sqrt( Gprime / (alpha^2 - 1) );
end

function blur_radius1 = estimate_image1_blur(blur_radius2, alpha, beta)
    blur_radius1(:,:) = alpha .* blur_radius2 + beta;
end

function im = remove_outliers(image, window_size)
%    mx = max(max(image));
%    mn = min(min(image));
%    range = mx - mn;

 %   v = reshape(image,1,size(image,1)*size(image,2));
 %   hist(v);
 %   pause;
 %   im = image;
    im = medfilt2(image, [window_size+1 window_size+1]);
end

function depth = depth_from_blur(m, c, blur_radius)
    top = blur_radius - c;
    depth_inverse = top ./ m;
    depth = 1 ./ depth_inverse;
end

%%
% camera calcs

function beta= calc_beta(m1, m2, c1, c2)
    beta = c1 - c2 * (m1 / m2);
end

function alpha = calc_alpha(m1, m2)
    alpha = m1 / m2;
end

function m = calc_m(image_data)
    m = -(image_data.aperture_size * image_data.corrected_lens_dis) / (2*sqrt(2));
end

function c = calc_c(image_data)
    negative_m = (image_data.aperture_size * image_data.corrected_lens_dis) / (2*sqrt(2));
    c = negative_m * (1/image_data.focal_length - 1/image_data.lens_dis);
end

function idata = calc_camera_params(image_data)
    image_data.pixels_per_mm = image_data.y_pixels_size / image_data.y_sensor_size;
    image_data.lens_dis = calc_lens_distance(image_data.focus_dis, image_data.focal_length);
    image_data.corrected_lens_dis = image_data.lens_dis;
    image_data.aperture_size = image_data.focal_length / image_data.f_number;
    image_data.pixels_per_mm = image_data.y_pixels_size / image_data.y_sensor_size;
    
    idata = image_data;
end

function lens_dis = calc_lens_distance(focus_dis, focal_length)
    %distance from focus object to image plane is:
    s1 = focus_dis - focal_length;
    
    s2 = 1 / (1/focal_length - 1/s1);
    lens_dis = s2;
end

%%
%misc functions

function show_shape(depth, colors)
    clf();
    hold on;  % don't fall!
    camproj('perspective');
    
    model_points = 50;
    
    %resize to make it faster
    if size(depth,1) > model_points
        depth = imresize( depth, [model_points NaN],  'bilinear');
        colors = imresize( colors, [model_points NaN],  'bilinear');
    end

    %{
    y = ones(size(depth,1)*size(depth,2));
    x = ones(size(depth,1)*size(depth,2));

    for j=1:size(depth, 1)
        for i=1:size(depth, 2)
            y( (j-1)*size(depth, 1) + i ) = j;
            x( (j-1)*size(depth, 1) + i ) = i;
        end
    end
    
    plot3(x', y', reshape(depth, size(depth,1)*size(depth,2), 1) , '.b');
%}    

    fprintf('starting 3d conversion\n');
    
    for y=1:size(depth, 1)
        for x=1:size(depth, 2)
            if depth(y,x) > 4000 || depth(y,x) < -2000
                continue;
            end
            c = double(reshape( colors(y,x,:), 1, 3)) ./ 255;
            plot3(x, y, depth(y,x), 'Color', c, 'Marker', '.', 'MarkerSize', 30);
            %[x y depth(y,x)];
        end
    end
    %zlim([-1000 3000])
end

function show_image(image, text)
    global show_images
    if show_images
        imshow(scale_values_for_display(image));
        title(text);
        pause;
    end
end

function print_matrix(m, name)
    global debug_on;
    
    if debug_on == 0
        return;
    end

    fprintf([name '\n']);

    if debug_on > 0
        fprintf('min=%f mean=%f max=%f \n', min(min(m)), mean(mean(m)), max(max(m)));
    end

    if debug_on > 1
        disp(m);
        fprintf('\n');
    end
    
    fprintf('\n');
end

function image_out = normalize_and_greyscale(image)
    image_out = image;
    if size(image,3) > 1
        image_out = rgb2gray(image_out);
    end
    
    image_out = double(image_out);
    m = mean(mean(image_out));
    image_out = image_out ./ mean(mean(image_out));
end

function image_out = scale_values_for_display(image)
    image_out = image;
    if size(image,3) > 1
        image_out = rgb2gray(image_out);
    end
    
    image_out = double(image_out);
    
    if min(min(image)) < 0
        image_out = image_out - min(min(image_out));
    end
    
    image_out = image_out ./ max(max(image_out));
end

function im = load_image_data(name)
    image_data = struct('image', {}, 'focus_dis', {}, 'f_number',{}, ...
        'focal_length',{}, 'y_pixel_size',{}, 'y_sensor_size',{}, ...
        'pixels_per_mm',{}, 'lens_dis',{}, 'corrected_lens_dis', {}, ...
        'aperture_size',{});
    
    global resize_size;

    image = imread([name '.png']);
    data = load([name '.txt']);
    image_data(1).image = image;
    if resize_size > 0
        image_data.image = imresize( image_data(1).image, [resize_size NaN],  'bilinear');
    end
    image_data(1).focus_dis = data(1)*1000;
    image_data(1).f_number = data(2);
    image_data(1).focal_length = data(3);
    image_data(1).y_pixels_size = data(4);
    image_data(1).y_sensor_size = data(5);
    
    image_data(1) = calc_camera_params(image_data(1));
    
    im = image_data(1);
end

function print_image_data(image_data)
    image_data
end

